#include <stdio.h>
#include <X11/Xlib.h>

extern KeySym K_LEFT,K_RIGHT,K_DOWN,K_ROTATE,K_BOTTOM,K_UP;

#define TOTAL 6

char message[TOTAL][26] =
{
	"Press key for MOVE LEFT",
	"Press key for MOVE RIGHT",
	"Press key for FAST DOWN",
	"Press key for ROTATE",
	"Press key for ROCK BOTTOM",
	"Press key for LEVEL UP"
};

KeySym keysym[TOTAL];
int max;

void PromptMessage(Display *disp,Window win,GC gc,int current)
{
	XClearWindow(disp,win);
	XDrawString(disp,win,gc,50,50,message[current],strlen(message[current]));
	XSync(disp,False);
}

void SaveKeysyms(Display *disp,Window win,GC gc,XFontStruct *font)
{
	FILE *fp;
	char filename[128];

	XUnmapWindow(disp,win);
	XDestroyWindow(disp,win);
	XFreeGC(disp,gc);
	XFreeFont(disp,font);
	XCloseDisplay(disp);

	if ((void *)getenv("HOME") == NULL)
	{
		fprintf(stderr,"Can't determine your home dir path!\n");
		exit(-1);
	}

	strcpy(filename,(char *)getenv("HOME"));
	if (filename[strlen(filename)-1] != '/') strcat(filename,"/");
	strcat(filename,".tetrisrc");

	fp=fopen(filename,"w");
	fprintf(fp,"LEFT=%u\n",keysym[0]);
	fprintf(fp,"RIGHT=%u\n",keysym[1]);
	fprintf(fp,"DOWN=%u\n",keysym[2]);
	fprintf(fp,"ROTATE=%u\n",keysym[3]);
	fprintf(fp,"BOTTOM=%u\n",keysym[4]);
	fprintf(fp,"UP=%u\n",keysym[5]);
	fclose(fp);
}

void customize_keyboard()
{
	Display *disp;
	Window win,root;
	int screen,white,black,current,i;
	GC gc;
	XFontStruct *font;
	XEvent event;

	if ((disp=XOpenDisplay(NULL)) == NULL)
	{
		perror("XOpenDisplay");
		exit(0);
	}

	if ((font=XLoadQueryFont(disp,"fixed")) == NULL)
	{
		perror("XLoadQueryFont");
		exit(0);
	}

	max=0;
	for (i=0; i<TOTAL; i++)
	{
		int test=XTextWidth(font,message[i],strlen(message[i]));
		if (test > max) max=test;
	}
	max+=100;

	root=DefaultRootWindow(disp);
	screen=DefaultScreen(disp);
	white=WhitePixel(disp,screen);
	black=BlackPixel(disp,screen);

	win=XCreateSimpleWindow(disp,root,0,0,max,100,1,white,white);
	XStoreName(disp,win,"Keyboard customization"); 
	XMapWindow(disp,win);
	gc=XCreateGC(disp,win,0,NULL);
	XSetForeground(disp,gc,black);
	XSetFont(disp,gc,font->fid);

	XSelectInput(disp,win,ExposureMask|KeyPressMask);

	XSync(disp,False);

	current=0;

	for (;;)
	{
		XNextEvent(disp,&event);

		switch (event.type)
		{
		case Expose:
			PromptMessage(disp,win,gc,current);
			break;

		case KeyPress:
			keysym[current++]=event.xkey.keycode;
			if (current == TOTAL)
			{
				SaveKeysyms(disp,win,gc,font);
				return;
			}
			PromptMessage(disp,win,gc,current);
			break;
		}
	}
}

void read_tetrisrc()
{
	FILE *fp;
	char filename[128],st[128];

	if ((void *)getenv("HOME") == NULL) return;
	strcpy(filename,(char *)getenv("HOME"));
	if (filename[strlen(filename)-1] != '/') strcat(filename,"/");
	strcat(filename,".tetrisrc");

	if ((fp=fopen(filename,"r")) == NULL)
	{
		customize_keyboard();
		fp=fopen(filename,"r");
	}
	while (fgets(st,128,fp))
	{
		if (st[0] == 0 || st[0] == '#') continue;

		if (strncmp(st,"LEFT=",5) == 0) sscanf(st+5,"%d",&K_LEFT); else
		if (strncmp(st,"RIGHT=",6) == 0) sscanf(st+6,"%d",&K_RIGHT); else
		if (strncmp(st,"DOWN=",5) == 0) sscanf(st+5,"%d",&K_DOWN); else
		if (strncmp(st,"ROTATE=",7) == 0) sscanf(st+7,"%d",&K_ROTATE); else
		if (strncmp(st,"BOTTOM=",7) == 0) sscanf(st+7,"%d",&K_BOTTOM); else
		if (strncmp(st,"UP=",3) == 0) sscanf(st+3,"%d",&K_UP);
	}
	fclose(fp);
}
